@@ -76,22 +76,22 @@
 /*	mail_stream_ctl() selectively overrides information that
 /*	was specified with mail_stream_file(); none of the attributes
 /*	are applicable for other mail stream types.  The arguments
-/*	are a list of (operation, value) pairs, terminated with
-/*	MAIL_STREAM_CTL_END.  The following lists the operation
-/*	codes and the types of the corresponding value arguments.
-/* .IP "MAIL_STREAM_CTL_QUEUE (char *)"
+/*	are a list macros with arguments, terminated with
+/*	CA_MAIL_STREAM_CTL_END which has none.  The following lists
+/*	the macros and the types of the corresponding arguments.
+/* .IP "CA_MAIL_STREAM_CTL_QUEUE(const char *)"
 /*	The argument specifies an alternate destination queue. The
 /*	queue file is moved to the specified queue before the call
 /*	returns. Failure to rename the queue file results in a fatal
 /*	error.
-/* .IP "MAIL_STREAM_CTL_CLASS (char *)"
+/* .IP "CA_MAIL_STREAM_CTL_CLASS(const char *)"
 /*	The argument specifies an alternate trigger class.
-/* .IP "MAIL_STREAM_CTL_SERVICE (char *)"
+/* .IP "CA_MAIL_STREAM_CTL_SERVICE(const char *)"
 /*	The argument specifies an alternate trigger service.
-/* .IP "MAIL_STREAM_CTL_MODE (int)"
+/* .IP "CA_MAIL_STREAM_CTL_MODE(int)"
 /*	The argument specifies alternate permissions that override
 /*	the permissions specified with mail_stream_file().
-/* .IP "MAIL_STREAM_CTL_DELAY (int)"
+/* .IP "CA_MAIL_STREAM_CTL_DELAY(int)"
 /*	Attempt to postpone initial delivery by advancing the queue
 /*	file modification time stamp by this amount.  This has
 /*	effect only within the deferred mail queue.
@@ -136,6 +136,7 @@
 #include <opened.h>
 #include <mail_params.h>
 #include <mail_stream.h>
+#include <mail_parm_split.h>
 
 /* Application-specific. */
 
@@ -154,7 +155,7 @@ void    mail_stream_cleanup(MAIL_STREAM *info)
     FREE_AND_WIPE(myfree, info->id);
     FREE_AND_WIPE(myfree, info->class);
     FREE_AND_WIPE(myfree, info->service);
-    myfree((char *) info);
+    myfree((void *) info);
 }
 
 #if defined(HAS_FUTIMES_AT)
@@ -377,11 +378,11 @@ static int mail_stream_finish_ipc(MAIL_STREAM *info, VSTRING *why)
      * Receive the peer's completion status.
      */
     if ((why && attr_scan(info->stream, ATTR_FLAG_STRICT,
-			  ATTR_TYPE_INT, MAIL_ATTR_STATUS, &status,
-			  ATTR_TYPE_STR, MAIL_ATTR_WHY, why,
+			  RECV_ATTR_INT(MAIL_ATTR_STATUS, &status),
+			  RECV_ATTR_STR(MAIL_ATTR_WHY, why),
 			  ATTR_TYPE_END) != 2)
 	|| (!why && attr_scan(info->stream, ATTR_FLAG_MISSING,
-			      ATTR_TYPE_INT, MAIL_ATTR_STATUS, &status,
+			      RECV_ATTR_INT(MAIL_ATTR_STATUS, &status),
 			      ATTR_TYPE_END) != 1))
 	status = CLEANUP_STAT_WRITE;
 
@@ -440,7 +441,7 @@ MAIL_STREAM *mail_stream_service(const char *class, const char *name)
 
     stream = mail_connect_wait(class, name);
     if (attr_scan(stream, ATTR_FLAG_MISSING,
-		  ATTR_TYPE_STR, MAIL_ATTR_QUEUEID, id_buf, 0) != 1) {
+		  RECV_ATTR_STR(MAIL_ATTR_QUEUEID, id_buf), 0) != 1) {
 	vstream_fclose(stream);
 	return (0);
     } else {
@@ -478,21 +479,21 @@ MAIL_STREAM *mail_stream_command(const char *command)
      * talking a Postfix-internal protocol there is no way we can tell what
      * is being executed except by duplicating a lot of existing code.
      */
-    export_env = argv_split(var_export_environ, ", \t\r\n");
+    export_env = mail_parm_split(VAR_EXPORT_ENVIRON, var_export_environ);
     while ((stream = vstream_popen(O_RDWR,
-				   VSTREAM_POPEN_COMMAND, command,
-				   VSTREAM_POPEN_EXPORT, export_env->argv,
-				   VSTREAM_POPEN_END)) == 0) {
+				   CA_VSTREAM_POPEN_COMMAND(command),
+				   CA_VSTREAM_POPEN_EXPORT(export_env->argv),
+				   CA_VSTREAM_POPEN_END)) == 0) {
 	msg_warn("fork: %m");
 	sleep(10);
     }
     argv_free(export_env);
     vstream_control(stream,
-		    VSTREAM_CTL_PATH, command,
-		    VSTREAM_CTL_END);
+		    CA_VSTREAM_CTL_PATH(command),
+		    CA_VSTREAM_CTL_END);
 
     if (attr_scan(stream, ATTR_FLAG_MISSING,
-		  ATTR_TYPE_STR, MAIL_ATTR_QUEUEID, id_buf, 0) != 1) {
+		  RECV_ATTR_STR(MAIL_ATTR_QUEUEID, id_buf), 0) != 1) {
 	if ((status = vstream_pclose(stream)) != 0)
 	    msg_warn("command \"%s\" exited with status %d", command, status);
 	return (0);
@@ -598,8 +599,8 @@ void    mail_stream_ctl(MAIL_STREAM *info, int op,...)
 
 	(void) mail_queue_path(new_path, new_queue, info->id);
 	info->queue = mystrdup(new_queue);
-	vstream_control(info->stream, VSTREAM_CTL_PATH, STR(new_path),
-			VSTREAM_CTL_END);
+	vstream_control(info->stream, CA_VSTREAM_CTL_PATH(STR(new_path)),
+			CA_VSTREAM_CTL_END);
 
 	if (sane_rename(saved_path, STR(new_path)) == 0
 	    || (mail_queue_mkdirs(STR(new_path)) == 0
